/* Copyright 2008-2014 Research Foundation State University of New York   */

/* This file is part of QUB Express.                                      */

/* QUB Express is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License as published by   */
/* the Free Software Foundation, either version 3 of the License, or      */
/* (at your option) any later version.                                    */

/* QUB Express is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License,     */
/* named LICENSE.txt, in the QUB Express program directory.  If not, see  */
/* <http://www.gnu.org/licenses/>.                                        */

/*
begin_html
<p>
 See also:
 <ul>
 </ul>
 <b>Up:</b> <a href="index.html">Index</a>
</p>
end_html
*/

#ifdef _WIN32
  #include <windows.h>
#else
  #include <stdlib.h>
  #define BOOL int
  #define TRUE 1
  #define FALSE 0
#endif

#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <cstdlib>

#include "qub_kalman.h"
#include "ublas_matrixutil.h"

using std::cout;
using std::cerr;
using std::endl;
using std::map;
using std::pair;

class qub_dotmap
{
public:
  int N;
  typedef map<float, pair<int, float> > dotmap;
  dotmap data; // x -> (i, y)
  
  void reset(int Ndata, float *xx, float *yy)
  {
    data.clear();
    for (int i=0; i<Ndata; ++i) {
      double key = xx[i];
      while ( data.find((float) key) != data.end() )
	key += rand() * 1.0 / RAND_MAX;
      data[(float) key] = pair<int, float>(i, yy[i]);
    }
  }
  int find(float x, float y, float rad)
  {
    int iBest = -1;
    float mBest = 1e10;
    for (dotmap::iterator ii=data.lower_bound(x-rad); (ii != data.end()) && (ii->first <= (x+rad)); ++ii) {
      float measure = (float) (fabs(x - ii->first) + fabs(y - ii->second.second));
      if ( measure < (2*rad) && measure < mBest ) {
	iBest = ii->second.first;
	mBest = measure;
      }
    }
    return iBest;
  }
  int in_rect(float x0, float y0, float x1, float y1, int *indices)
  {
    int count = 0;
    for (dotmap::iterator ii=data.begin(); ii != data.end(); ++ii)
      if ( (x0 <= ii->first) && (ii->first <= x1) &&
	   (y0 <= ii->second.second) && (ii->second.second <= y1) )
	indices[count++] = ii->second.first;
    return count;
  }
};
  

extern "C" QUBFAST_API void * qub_dotmap_create(int Ndata, float *xx, float *yy)
{
  qub_dotmap *dotmap = new qub_dotmap;
  dotmap->reset(Ndata, xx, yy);
  return dotmap;
}

extern "C" QUBFAST_API void qub_dotmap_free(void *dotmap)
{
  delete (qub_dotmap*) dotmap;
}

extern "C" QUBFAST_API void qub_dotmap_reset(void *dotmap, int Ndata, float *xx, float *yy)
{
  ((qub_dotmap *) dotmap)->reset(Ndata, xx, yy);
}

extern "C" QUBFAST_API int qub_dotmap_find(void *dotmap, float x, float y, float rad)
{
  return ((qub_dotmap *) dotmap)->find(x, y, rad);
}

extern "C" QUBFAST_API int qub_dotmap_in_rect(void *dotmap, float x0, float y0, float x1, float y1, int *indices)
{
  return ((qub_dotmap *) dotmap)->in_rect(x0, y0, x1, y1, indices);
}

